from unittest import mock
import unittest
import os
from webscrapbook.locales import I18N

root_dir = os.path.abspath(os.path.dirname(__file__))
test_root = os.path.join(root_dir, 'test_locales')
test_dirs = [
    os.path.join(test_root, 'test_general', 'host'),
    os.path.join(test_root, 'test_general', 'user'),
    ]

class TestI18N(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.maxDiff = 8192

    def test_init(self):
        # zh_CN, ch-CN, zh_cn, zh-cn should all work same
        i18n = I18N(test_dirs, 'zh_CN', 'messages')
        self.assertEqual([t.__file__ for t in i18n.translators], [
            os.path.join(test_root, 'test_general', 'host', 'zh_cn', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'en', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh_cn', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'en', 'messages.py'),
            ])

        i18n = I18N(test_dirs, 'zh-CN', 'messages')
        self.assertEqual([t.__file__ for t in i18n.translators], [
            os.path.join(test_root, 'test_general', 'host', 'zh_cn', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'en', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh_cn', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'en', 'messages.py'),
            ])

        i18n = I18N(test_dirs, 'zh_cn', 'messages')
        self.assertEqual([t.__file__ for t in i18n.translators], [
            os.path.join(test_root, 'test_general', 'host', 'zh_cn', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'en', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh_cn', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'en', 'messages.py'),
            ])

        i18n = I18N(test_dirs, 'zh-cn', 'messages')
        self.assertEqual([t.__file__ for t in i18n.translators], [
            os.path.join(test_root, 'test_general', 'host', 'zh_cn', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'en', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh_cn', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'en', 'messages.py'),
            ])

        # zh_tw => zh => en
        i18n = I18N(test_dirs, 'zh_TW', 'messages')
        self.assertEqual([t.__file__ for t in i18n.translators], [
            os.path.join(test_root, 'test_general', 'host', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'en', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'en', 'messages.py'),
            ])

        # no domain => defaults to 'messages'
        i18n = I18N(test_dirs, 'zh_TW')
        self.assertEqual([t.__file__ for t in i18n.translators], [
            os.path.join(test_root, 'test_general', 'host', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'host', 'en', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'zh', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'en', 'messages.py'),
            ])

        # en
        i18n = I18N(test_dirs, 'en')
        self.assertEqual([t.__file__ for t in i18n.translators], [
            os.path.join(test_root, 'test_general', 'host', 'en', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'en', 'messages.py'),
            ])

        # unknown lang should fallback to default
        i18n = I18N(test_dirs, 'wtf')
        self.assertEqual([t.__file__ for t in i18n.translators], [
            os.path.join(test_root, 'test_general', 'host', 'en', 'messages.py'),
            os.path.join(test_root, 'test_general', 'user', 'en', 'messages.py'),
            ])

    def test_call(self):
        i18n = I18N(test_dirs, 'zh_CN')
        self.assertEqual(i18n('test'), '测试 host')

        i18n = I18N(test_dirs, 'zh_TW')
        self.assertEqual(i18n('test'), '測試 host')

        i18n = I18N(test_dirs, 'en')
        self.assertEqual(i18n('test'), 'Test host')

    def test_call_missing(self):
        i18n = I18N(test_dirs, 'zh_TW')
        self.assertEqual(i18n('MyMessage'), 'MyMessage')

    def test_special(self):
        i18n = I18N([os.path.join(test_root, 'test_special')], 'zh_TW')
        self.assertEqual(i18n('@@ui_locale'), 'zh_tw')
        self.assertEqual(i18n('@@bidi_dir'), 'ltr')
        self.assertEqual(i18n('@@bidi_reversed_dir'), 'rtl')
        self.assertEqual(i18n('@@bidi_start_edge'), 'left')
        self.assertEqual(i18n('@@bidi_end_edge'), 'right')

        i18n = I18N([os.path.join(test_root, 'test_special')], 'ar')
        self.assertEqual(i18n('@@ui_locale'), 'ar')
        self.assertEqual(i18n('@@bidi_dir'), 'rtl')
        self.assertEqual(i18n('@@bidi_reversed_dir'), 'ltr')
        self.assertEqual(i18n('@@bidi_start_edge'), 'right')
        self.assertEqual(i18n('@@bidi_end_edge'), 'left')

if __name__ == '__main__':
    unittest.main()



"""Server backend of WebScrapBook toolkit.
"""
import os
import webbrowser
from threading import Thread

# dependency
from werkzeug.serving import WSGIRequestHandler, make_server

# this package
from . import Config
from .app import make_app
from .util import is_nullhost

def serve(root, browse=None):
    config = Config()
    config.load(root)

    # set params
    host = config['server']['host'] or '127.0.0.1'
    port = config['server']['port']
    ssl_on = config['server']['ssl_on']
    ssl_key = config['server']['ssl_key'] if ssl_on else None
    ssl_cert = config['server']['ssl_cert'] if ssl_on else None
    scheme = 'https' if ssl_on else 'http'

    if ssl_key:
        ssl_key = os.path.abspath(os.path.join(root, ssl_key))

    if ssl_cert:
        ssl_cert = os.path.abspath(os.path.join(root, ssl_cert))

    host2 = f'[{host}]' if ':' in host else host
    host3 = 'localhost' if is_nullhost(host) else host2
    port2 = '' if (not ssl_on and port == 80) or (ssl_on and port == 443) else ':' + str(port)

    # prepare server
    srv = make_server(
        host=host,
        port=port,
        app=make_app(root, config),
        threaded=True,
        processes=1,
        ssl_context=((ssl_cert, ssl_key) if ssl_cert and ssl_key
                else 'adhoc' if ssl_on else None),
        request_handler=RequestHandler,
        )

    srv.log('info', 'WebScrapBook server starting up...')
    srv.log('info', f'Document Root: {os.path.abspath(root)}')
    srv.log('info', f'Listening on {scheme}://{host2}:{port}')
    srv.log('info', 'Hit Ctrl-C to shutdown.')

    # launch browser
    if browse is None:
        browse = config['server']['browse']

    if browse:
        base = config['app']['base'].rstrip('/')
        index = config['app']['index'].lstrip('/')
        path = base + (('/' + index) if index else '')
        url = f'{scheme}://{host3}{port2}{path}'
        srv.log('info', f'Launching browser at {url} ...')
        try:
            browser = webbrowser.get(config['browser']['command'] or None)
        except webbrowser.Error as exc:
            srv.log('error', f'Error: {exc}')
        else:
            thread = Thread(target=browser.open, args=[url], daemon=True)
            thread.start()

    # start server
    srv.serve_forever()


class RequestHandler(WSGIRequestHandler):
    protocol_version = "HTTP/1.1"
